% Code quantitative image of diole through an optical microscope
% S.KHADIR et al. Institut Fresnel, Marseille (France)

%% inputs

%n_b        refractive index of the background medium of the dipole
%n          refractive index of the microscope coverslip and the imersion medium
%M          Microscope magnification
%NA         Numerical aperture of the microscope objective
%lambda     illumination wavelength  in m
%I          Intensity of illumination light (W/m^2)
%r          radius of the particle in m
%epsm       dielectric function of the nanoparticle
%alpha      polar angle of the dipole moment
%betha      azimuthal angle of the dipole moment
%zQ         position of the dipole according to the interface plane (m)
%zo         position of the focal plane according to the interface plane (m)
%Npx        pixel number

clear 

%% Microscope paramaters
nb = 1.5;            % refractive index of the background medium of the dipole
n = 1.5;             % refractive index of the microscope coverslip and the imersion medium
M = -100;            % Microscope magnification 
NA = 0.9;            % Numerical aperture of the microscope objective

if NA>n
    error('NA should be smaller than n')
end

%% excitation field
lambda = 590e-9;             % illumination wavelength (m)
I = 1e9;                     % Intensity of illumination light (W/m^2)

eps0 = 1e-9/(36*pi);         % vacuum permittivity
c = 299792458;               % speed of light
k0 = 2*pi/lambda;            % wavevector in vacuum
tr = 2*nb/(nb+n);            % transmission coefficient through the interface
e0 = sqrt(2*I/(nb*c*eps0));  % incident electic field
eex = tr*e0;                 % excitation electric field

%% Dipole created by a nanosphere
r = 37e-9;                                                                 % radius of the sphere (m)
epsm = epsmRead(lambda,'Au');                                              % dielectric function of the nanoparticle

a = Polarizability(lambda,r,nb,'Au');                                      % Mie computation of the nanoparticle polarizability                                                        

% Polarization of the dipole (Ex: alpha = pi/2, betha = 0 --> the dipole
% oriented according to x direction)
alpha = pi/2;                                                              % polar angle of the dipole moment
betha = 0;                                                                 % azimuthal angle of the dipole moment 
Q = eps0*a*eex*[sin(alpha)*cos(betha);sin(alpha)*sin(betha);cos(alpha)];   % dipole moment of the nanoparticle

%% defocus and position of the dipole
zQ = 0e-9;  % position of the dipole according to the interface plane (m)
zo = 0e-9; % position of the focal plane according to the interface plane (m)

%% Computation of the scatred field at fourier plane
Npx = 100;                         % pixel number
factor = 4;                        % artificially increase the size of the image by factor 
kmax = nb*k0; 
dk = kmax/(Npx);                   % pixel size in fourier space
kxx = -factor*kmax:dk:kmax*factor; % x component of wavevector
kyy = -factor*kmax:dk:kmax*factor; % y component of wavevector

% initialization 
Eka = zeros(length(kxx),length(kyy));
Ekxa = zeros(length(kxx),length(kyy));
Ekya = zeros(length(kxx),length(kyy));
Ekza = zeros(length(kxx),length(kyy));
Ekprime = zeros(length(kxx),length(kyy));
Ekprimex = zeros(length(kxx),length(kyy));
Ekprimey = zeros(length(kxx),length(kyy));
Ekprimez = zeros(length(kxx),length(kyy));
z = [0;0;1];

 for ikx=1:numel(kxx)
     kx=kxx(ikx);
     kx0=kx/kmax; 
     kprimex=kx/M;
     kprimex0=kprimex/k0;
     
     for iky=1:numel(kyy)
         ky=kyy(iky);
         ky0=ky/kmax;
         kprimey=ky/M;
         kprimey0=kprimey/k0;
        
         if sqrt(kx*kx+ky*ky) <= NA*k0     % cutoff filter; taking into account the numerical apperture of the objective
             
         ga=sqrt(n^2*k0^2-kx^2-ky^2);
         ga0=sqrt(n^2*k0^2-kx^2-ky^2)/(n*k0);
         
         gab=sqrt(nb^2*k0^2-kx^2-ky^2);
         gab0=sqrt(nb^2*k0^2-kx^2-ky^2)/(nb*k0);
         
         kv = [kx0;ky0;ga0];               % wavevector in the medium of refractive index n
         kvb = [kx0;ky0;gab0];             % wavevector in the medium of refractive index nb
         
         % transmission coefficiens for s and p polarizations
         ts=2*gab/(gab+ga);
         tp=2*nb*n*gab/(n^2*gab+nb^2*ga);
        
         % s and p basis in the medium of refractive index n
         s1=cross(z,kv);
         if (s1==0) 
             s=[1;0;0];
             p=[0;0;0];
         else
         s=cross(z,kv)/norm(s1);
         p1=cross(kv,s);
         p=cross(kv,s)/norm(p1);
         end
         
         % s and p basis in the medium of refractive index nb
         sb1=cross(z,kvb);
         if (sb1==0) 
             sb=[1;0;0];
             pb=[0;0;0];
         else
         sb=cross(z,kvb)/norm(sb1);
         pb1=cross(kvb,sb);
         pb=cross(kvb,sb)/norm(pb1);
         end
         
         Q1 = Q-dot(conj(kvb),Q)*kvb; % radiated field of the dipole
         Qs = ts*dot(conj(Q1),sb)*s;  % transmission of s component of the radiated field
         Qp = tp*dot(conj(Q1),pb)*p;  % transmission of p component of the radiated field
         
         % planewave expantion of the radiated electric field 
         Eka = 1i*k0*k0/(8*pi*pi*eps0)*(1/gab)*(Qs+Qp)*exp(-1i*gab*zQ+1i*ga*zo); 
         Ekxa(iky,ikx) = Eka(1);
         Ekya(iky,ikx) = Eka(2);
         Ekza(iky,ikx) = Eka(3);
         
         % Computation of the electric field afeter rotation
         gaprime = sqrt(k0^2-kprimex^2-kprimey^2);
         gaprime0 = sqrt(k0^2-kprimex^2-kprimey^2)/k0;
         kvprime = [kprimex0;kprimey0;gaprime0];
         
         pprime1 = cross(kvprime,s);
         pprime = cross(kvprime,s)/norm(pprime1);
  
         Ekprime = sqrt(ga/gaprime)*(dot(conj(Eka),s)*s+dot(conj(Eka),p)*pprime);
          
         Ekprimex(iky,ikx) = Ekprime(1);
         Ekprimey(iky,ikx) = Ekprime(2);
         Ekprimez(iky,ikx) = Ekprime(3);
         end    
     end           
 end
 
%% Print Field at fourier plane 
IEkprime = abs(Ekprimex).^2+abs(Ekprimey).^2+abs(Ekprimez).^2;  %Intensity of the electric field in fourier space
figure,imagesc(IEkprime),colormap(gray),colorbar
title 'Intensity of the scattered field at Fourier plane'
figure,imagesc(angle(Ekprimex)),colorbar
title 'Phase of the scattered field at Fourier plane'

%% Computation of the scatred field at image plane 
Expp = (1/M)*fftshift(fft2(ifftshift(Ekprimex)))*dk*dk; 
Eypp = (1/M)*fftshift(fft2(ifftshift(Ekprimey)))*dk*dk;
Ezpp = (1/M)*fftshift(fft2(ifftshift(Ekprimez)))*dk*dk;

IErpp = abs(Expp).^2+abs(Eypp).^2+abs(Ezpp).^2;
figure,imagesc(IErpp),colormap(gray),colorbar
title 'Intensity of the scattered field at image plane'

%% Pixel size 
dr = 2*pi/((2*factor*Npx+1)*dk); %pixel size at the image plane (m)

%% Excitation field image plane
eexim = tr/M*sqrt(n)*e0*exp(1i*n*k0*zo); 

%% Total field image plane
Eixtot = eexim+Expp;
Phase = angle(Eixtot);
IEitot = abs(Eixtot).^2+abs(Eypp).^2+abs(Ezpp).^2;

%% Normalized total field image plane 
norExtot = Eixtot./eexim;
norEytot = Eypp./eexim;
norEztot = Ezpp./eexim;
norEtot = norExtot+norEytot+norEztot;
T = abs(norExtot).^2+abs(norEytot).^2+abs(norEztot).^2; % Transmittance
Ph = angle(norExtot); % phase
OPD = Ph./k0; % Optical path difference

%% polarizability calcualtion 
sqtT = sqrt(T);
expPh = exp(1i*Ph);
alpha = 1i*lambda*(nb+n)/(2*pi)*(1-sqtT.*expPh);
alpha3D = sum(alpha(:))*dr*dr;

%% Print images
crop = 100;
OPDcrop = OPD((length(kxx)+1)/2-crop:(length(kxx)+1)/2+crop,(length(kxx)+1)/2-crop:(length(kxx)+1)/2+crop);
Tcrop = T((length(kxx)+1)/2-crop:(length(kxx)+1)/2+crop,(length(kxx)+1)/2-crop:(length(kxx)+1)/2+crop);
Tprofile = Tcrop(101,:);
OPDprofile = OPDcrop(101,:);

hfig = figure;
pos = get(hfig,'position');
set(hfig,'position',pos.*[1 1 1 4]);
subplot(4,1,1),imagesc(Tcrop),colormap(gray),colorbar,axis equal
title('normalized total intensity image plane')
subplot(4,1,2),plot(Tprofile);
title('normalized total intensity profile')
subplot(4,1,3),imagesc(OPDcrop),colormap(gca,'parula'),colorbar,axis equal
title('OPD image plane (m)')
subplot(4,1,4),plot(OPDprofile)
title('OPD profile (m)')

